==> spread-5.0.0.54-py3-none-any.whl: spread_core/bam/handling/scheduler.py <==
import datetime
import logging
from threading import Timer

from spread_core.tools.settings import config, TASKS
from . import DateTrigger, Location
from .handler import Handler


class Scheduler(Handler):
    triggers_path = property(fget=lambda self: config[TASKS])

    def __init__(self, project_id, _mqttc):
        super(Scheduler, self).__init__(project_id, _mqttc)
        self.timer = None

    def on_ready(self):
        self.on_timer()

    def parse(self, data):
        for datum in data:
            try:
                location = Location.deserialize(datum)
                if location.enabled:
                    self.triggers.update(location.triggers)
            except BaseException as ex:
                logging.exception(ex)

    def on_timer(self):
        try:
            triggers = sorted(self.triggers)
            if len(triggers) > 0:
                trigger = triggers[0]
                interval = trigger.time_left + 1
                logging.info(f'SET TIMER to {datetime.timedelta(seconds=interval)} sec FOR {trigger}')
                if self.timer:
                    self.timer.cancel()
                self.timer = Timer(interval=interval, function=self.exec_task, args=[trigger])
                self.timer.start()
            else:
                logging.info('COMPLETE')
                # self.finish()
        except BaseException as ex:
            logging.exception(ex)

    def exec_task(self, _trigger: DateTrigger):
        for trigger, actions in self.triggers.items():
            if trigger == _trigger:
                logging.info(f'EXECUTE {_trigger}:')
                for action in actions:
                    try:
                        action.execute(self.publish)
                    except BaseException as ex:
                        logging.exception(ex)

        self.on_timer()

    def on_exit(self):
        super(Scheduler, self).on_exit()
        if self.timer:
            self.timer.cancel()
            self.timer = None

==> spread-5.1.0.48-py3-none-any.whl: spread_core/bam/handling/scheduler.py <==
import datetime
import logging
from threading import Timer

from spread_core.tools.settings import config, TASKS
from . import DateTrigger, Location
from .handler import Handler


class Scheduler(Handler):
    triggers_path = property(fget=lambda self: config[TASKS])

    def __init__(self, project_id, _mqttc):
        super(Scheduler, self).__init__(project_id, _mqttc)
        self.timer = None

    def on_ready(self):
        self.on_timer()

    def parse(self, data):
        for datum in data:
            try:
                location = Location.deserialize(datum)
                if location.enabled:
                    self.triggers.update(location.triggers)
            except BaseException as ex:
                logging.exception(ex)

    def on_timer(self):
        try:
            triggers = sorted(self.triggers)
            if len(triggers) > 0:
                trigger = triggers[0]
                interval = trigger.time_left + 1
                logging.info(f'SET TIMER to {datetime.timedelta(seconds=interval)} sec FOR {trigger}')
                if self.timer:
                    self.timer.cancel()
                self.timer = Timer(interval=interval, function=self.exec_task, args=[trigger])
                self.timer.start()
            else:
                logging.info('COMPLETE')
                # self.finish()
        except BaseException as ex:
            logging.exception(ex)

    def exec_task(self, _trigger: DateTrigger):
        for trigger, actions in self.triggers.items():
            if trigger == _trigger:
                logging.info(f'EXECUTE {_trigger}:')
                for action in actions:
                    try:
                        action.execute(self.publish, session_id=self.sessionID)
                    except BaseException as ex:
                        logging.exception(ex)

        self.on_timer()

    def on_exit(self):
        super(Scheduler, self).on_exit()
        if self.timer:
            self.timer.cancel()
            self.timer = None
